.PinCode {
  text-align: center;
  background-color: white;
  min-height: 100vh;
  display: flex;
  flex-direction: column;
  align-items: center;
  justify-content: center;
  font-size: calc(10px + 2vmin);
  color: #282c34;
}

.Button {
  background-color: #ffffff;
  border: none;
  color: #888888;
  display: inline-block;
  font-size: 32px;
  cursor:pointer;
}

.Pop { animation: pop 0.5s; }
@keyframes pop {
  10% {transform: scale(0.8);}
}

.FadeIn { animation: fadeIn 2s; }

@keyframes fadeIn {
  0% { opacity: 0; }
  100% { opacity: 1; }
}
