import React from 'react';
import ReactDOM from 'react-dom/client';
import './index.css';
import { PinCode } from './PinCode';

const root = ReactDOM.createRoot(document.getElementById('pin_code'));
root.render(
  <PinCode />
);
